@@ -50,6 +50,8 @@ class NapariSTPT:
 
         self.align_x = None
         self.align_y = None
+        self.corrected_align_x = None
+        self.corrected_align_y = None
         self.ds1 = None
         self.im1 = None
         self.im32 = None
@@ -57,6 +59,7 @@ class NapariSTPT:
         #self.movie = None
         self.optical_slices = None
         self.nr_optical_slices = None
+        self.optical_slices_available = None
         self.shape = None
 
         self.crop_start_ratio_x = None
@@ -71,6 +74,8 @@ class NapariSTPT:
         self.end_slice = None
 
         self.image_translation = None
+        self.m_slice_spacing = None
+        self.slice_spacing = None
         
     def CropOriginal(self, volume):
 
@@ -93,10 +98,7 @@ class NapariSTPT:
         return volume
         
     def Load(self, text):
-        # global self.ds1
-        # global ds
-        # global bscale
-        # global bzero
+
         try:
             self.viewer.layers.remove('C1')
             del self.aligned_1
@@ -172,11 +174,12 @@ class NapariSTPT:
             self.slice_names = self.ds1.attrs['cube_reg']['slice']
             self.align_x = self.ds1.attrs['cube_reg']['abs_dx']
             self.align_y = self.ds1.attrs['cube_reg']['abs_dy']
-            print(f"self.align_x length {len(self.align_x)}")
-            print(f"self.align_x {self.align_x}")
-            print(f"self.align_y {self.align_y}")
+            #print(f"self.align_x length {len(self.align_x)}")
+            #print("")
+            #print(f"self.align_x {self.align_x}")
+            #print(f"self.align_y {self.align_y}")
+            #print("")
         else:
-            
             bscale = self.ds1['S001'].attrs['bscale']
             bzero = self.ds1['S001'].attrs['bzero']
 
@@ -203,9 +206,9 @@ class NapariSTPT:
             index = 0
         
         if self.old_method:
-            print("loading at resolution {}".format(resolution))
+            #print("loading at resolution {}".format(resolution))
             #index = self.comboBoxResolution.currentIndex()
-            print("group: " + self.ds1.attrs["multiscale"]['datasets'][index]['path'])
+            #print("group: " + self.ds1.attrs["multiscale"]['datasets'][index]['path'])
             gr = self.ds1.attrs["multiscale"]['datasets'][index]['path']
             ds = xr.open_zarr(self.search_folder.text() + str(self.comboBoxPath.currentText())+'/mos.zarr', group=gr)
         else:
@@ -216,34 +219,65 @@ class NapariSTPT:
             print(gr)
             ds = xr.open_zarr(self.search_folder.text() + str(self.comboBoxPath.currentText())+'/mos.zarr', group=gr)
 
-        self.optical_slices = len(ds.z)
+        self.optical_slices_available = len(ds.z)
+        self.optical_slices = self.optical_slices_available
         print(f"optical_slices available: {self.optical_slices}")
         
+        self.slice_spacing = float(self.m_slice_spacing.text())
+
         if self.optical_slices > 1:
             optical_section_spacing = self.ds1['S001'].attrs['raw_meta'][0][0]['zres']
             print(f"optical_slices zres: {optical_section_spacing}")
-            expected_nr_of_slices = round(15.0 / optical_section_spacing)
+            expected_nr_of_slices = round(self.slice_spacing / optical_section_spacing)
             if self.optical_slices > expected_nr_of_slices:
                 self.optical_slices = expected_nr_of_slices
 
         print(f"optical_slices used: {self.optical_slices}")
 
-        
+        self.corrected_align_x = []
+        for index, value in enumerate(self.align_x):
+            if (index % self.optical_slices_available) < self.optical_slices:
+                self.corrected_align_x.append(value)
+
+        #self.align_x = self.corrected_align_x
+        #print(f"self.align_x shape {len(self.corrected_align_x)}")
+
+        self.corrected_align_y = []
+        for index, value in enumerate(self.align_y):
+            if (index % self.optical_slices_available) < self.optical_slices:
+                self.corrected_align_y.append(value)
+
+        #self.align_y = corrected_align_y
+        print(f"align_y  {self.align_y}")
+        print("")
+        print(f"corrected_align_y {self.corrected_align_y}")
+        print("")
+
+        for i in range(0,self.optical_slices):
+            self.corrected_align_y[i::self.optical_slices] = self.corrected_align_y[::self.optical_slices]
+            
+        print(f"corrected_align_y {self.corrected_align_y}")
+        print("")
 
         with napari.gui_qt():
             if self.cb_C1.isChecked():
                 print("loading C1")
-                volume_1_temp = (ds.sel(channel=1, type='mosaic', z=0).to_array().data * bscale + bzero).astype(dtype=np.float32)
+
+                volume_1_temp = (ds.sel(channel=1, type='mosaic', z=0).to_array(
+                ).data * bscale + bzero).astype(dtype=np.float32)
 
                 if self.start_slice.text() == "":
                     start_slice_number = 0
+                    chop_bottom = 0
                 else:
-                    start_slice_number = int(self.start_slice.text())
-
+                    start_slice_number = int(math.floor(float(self.start_slice.text())/float(self.optical_slices)))
+                    chop_bottom = int(self.start_slice.text()) - (self.optical_slices * start_slice_number) 
                 if self.end_slice.text() == "":
                     end_slice_number = volume_1_temp.shape[0]-1
+                    chop_top = 0
                 else:
-                    end_slice_number = int(self.end_slice.text())
+                    end_slice_number = int(math.floor(float(self.end_slice.text())/float(self.optical_slices)))
+                    chop_top = (self.optical_slices * (end_slice_number + 1) -1) - int(self.end_slice.text()) 
 
                 number_of_slices = end_slice_number - start_slice_number + 1
 
@@ -266,18 +300,18 @@ class NapariSTPT:
                 volume_1[0::self.optical_slices, :, :] = volume_1_temp[start_slice_number:end_slice_number+1, start_y:start_y+size_y, start_z:start_z+size_z]
 
                 for optical_slice in range(1, self.optical_slices):
-                    print(f"loading optical slice {optical_slice}")
-                    volume_1_temp = (ds.sel(channel=1, type='mosaic', z=optical_slice).to_array().data * bscale + bzero).astype(dtype=np.float32)
-                    volume_1[optical_slice::self.optical_slices, :, :] = volume_1_temp[start_slice_number:end_slice_number+1,start_y:start_y+size_y, start_z:start_z+size_z]
+                    volume_1_temp = (ds.sel(channel=1, type='mosaic', z=optical_slice).to_array(
+                    ).data * bscale + bzero).astype(dtype=np.float32)
+                    volume_1[optical_slice::self.optical_slices, :, :] = volume_1_temp[start_slice_number:end_slice_number+1, start_y:start_y+size_y, start_z:start_z+size_z]
 
-                self.aligned_1 = volume_1
+                # self.aligned_2 = volume_2
 
                 print("aligning C1")
-                self.aligned_1 = self.Align(volume_1, resolution, output_resolution, start_slice_number)
-                
+                self.aligned_1 = self.Align(volume_1, resolution, output_resolution, start_slice_number*self.optical_slices)
+                #self.aligned_1 = volume_1
+                self.aligned_1 = self.aligned_1[chop_bottom:self.aligned_1.shape[0]-chop_top,:,:]
                 self.shape = self.aligned_1.shape
-                self.viewer.add_image([self.aligned_1], name='C1', scale=(
-                    15/self.optical_slices, output_resolution, output_resolution), blending='additive', colormap='bop purple')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+                self.viewer.add_image([self.aligned_1], name='C1', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='bop purple')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
 
             if self.cb_C2.isChecked():
                 print("loading C2")
@@ -287,13 +321,16 @@ class NapariSTPT:
 
                 if self.start_slice.text() == "":
                     start_slice_number = 0
+                    chop_bottom = 0
                 else:
-                    start_slice_number = int(self.start_slice.text())
-
+                    start_slice_number = int(math.floor(float(self.start_slice.text())/float(self.optical_slices)))
+                    chop_bottom = int(self.start_slice.text()) - (self.optical_slices * start_slice_number) 
                 if self.end_slice.text() == "":
                     end_slice_number = volume_2_temp.shape[0]-1
+                    chop_top = 0
                 else:
-                    end_slice_number = int(self.end_slice.text())
+                    end_slice_number = int(math.floor(float(self.end_slice.text())/float(self.optical_slices)))
+                    chop_top = (self.optical_slices * (end_slice_number + 1) -1) - int(self.end_slice.text()) 
 
                 number_of_slices = end_slice_number - start_slice_number + 1
 
@@ -312,41 +349,41 @@ class NapariSTPT:
                     start_y = 0
                     start_z = 0
 
-                volume_2 = np.zeros(
-                    (self.optical_slices*number_of_slices, size_y, size_z), dtype=np.float32)
-                volume_2[0::self.optical_slices, :, :] = volume_2_temp[start_slice_number:end_slice_number+1,
-                                                                       start_y:start_y+size_y, start_z:start_z+size_z]
+                volume_2 = np.zeros((self.optical_slices*number_of_slices, size_y, size_z), dtype=np.float32)
+                volume_2[0::self.optical_slices, :, :] = volume_2_temp[start_slice_number:end_slice_number+1, start_y:start_y+size_y, start_z:start_z+size_z]
 
                 for optical_slice in range(1, self.optical_slices):
-                    print(f"loading optical slice {optical_slice}")
                     volume_2_temp = (ds.sel(channel=2, type='mosaic', z=optical_slice).to_array(
                     ).data * bscale + bzero).astype(dtype=np.float32)
-                    volume_2[optical_slice::self.optical_slices, :, :] = volume_2_temp[start_slice_number:end_slice_number+1,
-                                                                                       start_y:start_y+size_y, start_z:start_z+size_z]
+                    volume_2[optical_slice::self.optical_slices, :, :] = volume_2_temp[start_slice_number:end_slice_number+1, start_y:start_y+size_y, start_z:start_z+size_z]
 
                 # self.aligned_2 = volume_2
 
                 print("aligning C2")
-                self.aligned_2 = self.Align(
-                    volume_2, resolution, output_resolution, start_slice_number)
+                self.aligned_2 = self.Align(volume_2, resolution, output_resolution, start_slice_number*self.optical_slices)
+                #self.aligned_2 = volume_2
+                self.aligned_2 = self.aligned_2[chop_bottom:self.aligned_2.shape[0]-chop_top,:,:]
                 self.shape = self.aligned_2.shape
-                self.viewer.add_image([self.aligned_2], name='C2', scale=(
-                    15.0/self.optical_slices, output_resolution, output_resolution), blending='additive', colormap='red')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+                self.viewer.add_image([self.aligned_2], name='C2', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='red')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
 
             if self.cb_C3.isChecked():
                 print("loading C3")
+
                 volume_3_temp = (ds.sel(channel=3, type='mosaic', z=0).to_array(
                 ).data * bscale + bzero).astype(dtype=np.float32)
 
                 if self.start_slice.text() == "":
                     start_slice_number = 0
+                    chop_bottom = 0
                 else:
-                    start_slice_number = int(self.start_slice.text())
-
+                    start_slice_number = int(math.floor(float(self.start_slice.text())/float(self.optical_slices)))
+                    chop_bottom = int(self.start_slice.text()) - (self.optical_slices * start_slice_number) 
                 if self.end_slice.text() == "":
                     end_slice_number = volume_3_temp.shape[0]-1
+                    chop_top = 0
                 else:
-                    end_slice_number = int(self.end_slice.text())
+                    end_slice_number = int(math.floor(float(self.end_slice.text())/float(self.optical_slices)))
+                    chop_top = (self.optical_slices * (end_slice_number + 1) -1) - int(self.end_slice.text())
 
                 number_of_slices = end_slice_number - start_slice_number + 1
 
@@ -365,41 +402,41 @@ class NapariSTPT:
                     start_y = 0
                     start_z = 0
 
-                volume_3 = np.zeros(
-                    (self.optical_slices*number_of_slices, size_y, size_z), dtype=np.float32)
-                volume_3[0::self.optical_slices, :, :] = volume_3_temp[start_slice_number:end_slice_number+1,
-                                                                       start_y:start_y+size_y, start_z:start_z+size_z]
+                volume_3 = np.zeros((self.optical_slices*number_of_slices, size_y, size_z), dtype=np.float32)
+                volume_3[0::self.optical_slices, :, :] = volume_3_temp[start_slice_number:end_slice_number+1, start_y:start_y+size_y, start_z:start_z+size_z]
 
                 for optical_slice in range(1, self.optical_slices):
-                    print(f"loading optical slice {optical_slice}")
                     volume_3_temp = (ds.sel(channel=3, type='mosaic', z=optical_slice).to_array(
                     ).data * bscale + bzero).astype(dtype=np.float32)
-                    volume_3[optical_slice::self.optical_slices, :, :] = volume_3_temp[start_slice_number:end_slice_number+1,
-                                                                                       start_y:start_y+size_y, start_z:start_z+size_z]
+                    volume_3[optical_slice::self.optical_slices, :, :] = volume_3_temp[start_slice_number:end_slice_number+1, start_y:start_y+size_y, start_z:start_z+size_z]
 
                 # self.aligned_3 = volume_3
 
                 print("aligning C3")
-                self.aligned_3 = self.Align(
-                    volume_3, resolution, output_resolution, start_slice_number)
+                self.aligned_3 = self.Align(volume_3, resolution, output_resolution, start_slice_number*self.optical_slices)
+                #self.aligned_3 = volume_3
+                self.aligned_3 = self.aligned_3[chop_bottom:self.aligned_3.shape[0]-chop_top,:,:]
                 self.shape = self.aligned_3.shape
-                self.viewer.add_image([self.aligned_3], name='C3', scale=(
-                    15/self.optical_slices, output_resolution, output_resolution), blending='additive', colormap='green')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+                self.viewer.add_image([self.aligned_3], name='C3', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='green')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
 
             if self.cb_C4.isChecked():
                 print("loading C4")
+
                 volume_4_temp = (ds.sel(channel=4, type='mosaic', z=0).to_array(
                 ).data * bscale + bzero).astype(dtype=np.float32)
 
                 if self.start_slice.text() == "":
                     start_slice_number = 0
+                    chop_bottom = 0
                 else:
-                    start_slice_number = int(self.start_slice.text())
-
+                    start_slice_number = int(math.floor(float(self.start_slice.text())/float(self.optical_slices)))
+                    chop_bottom = int(self.start_slice.text()) - (self.optical_slices * start_slice_number) 
                 if self.end_slice.text() == "":
                     end_slice_number = volume_4_temp.shape[0]-1
+                    chop_top = 0
                 else:
-                    end_slice_number = int(self.end_slice.text())
+                    end_slice_number = int(math.floor(float(self.end_slice.text())/float(self.optical_slices)))
+                    chop_top = (self.optical_slices * (end_slice_number + 1) -1) - int(self.end_slice.text())
 
                 number_of_slices = end_slice_number - start_slice_number + 1
 
@@ -418,26 +455,22 @@ class NapariSTPT:
                     start_y = 0
                     start_z = 0
 
-                volume_4 = np.zeros(
-                    (self.optical_slices*number_of_slices, size_y, size_z), dtype=np.float32)
-                volume_4[0::self.optical_slices, :, :] = volume_4_temp[start_slice_number:end_slice_number+1,
-                                                                       start_y:start_y+size_y, start_z:start_z+size_z]
+                volume_4 = np.zeros((self.optical_slices*number_of_slices, size_y, size_z), dtype=np.float32)
+                volume_4[0::self.optical_slices, :, :] = volume_4_temp[start_slice_number:end_slice_number+1, start_y:start_y+size_y, start_z:start_z+size_z]
 
                 for optical_slice in range(1, self.optical_slices):
-                    print(f"loading optical slice {optical_slice}")
                     volume_4_temp = (ds.sel(channel=4, type='mosaic', z=optical_slice).to_array(
                     ).data * bscale + bzero).astype(dtype=np.float32)
-                    volume_4[optical_slice::self.optical_slices, :, :] = volume_4_temp[start_slice_number:end_slice_number+1,
-                                                                                       start_y:start_y+size_y, start_z:start_z+size_z]
+                    volume_4[optical_slice::self.optical_slices, :, :] = volume_4_temp[start_slice_number:end_slice_number+1, start_y:start_y+size_y, start_z:start_z+size_z]
 
                 # self.aligned_4 = volume_4
 
                 print("aligning C4")
-                self.aligned_4 = self.Align(
-                    volume_4, resolution, output_resolution, start_slice_number)
+                self.aligned_4 = self.Align(volume_4, resolution, output_resolution, start_slice_number*self.optical_slices)
+                #self.aligned_4 = volume_4
+                self.aligned_4 = self.aligned_4[chop_bottom:self.aligned_4.shape[0]-chop_top,:,:]
                 self.shape = self.aligned_4.shape
-                self.viewer.add_image([self.aligned_4], name='C4', scale=(
-                    15/self.optical_slices, output_resolution, output_resolution), blending='additive', colormap='bop blue')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+                self.viewer.add_image([self.aligned_4], name='C4', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='bop blue')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
 
             # create naparimovie object
             #if self.movie is None:
@@ -615,8 +648,8 @@ class NapariSTPT:
         aligned = np.zeros(size, dtype=np.float32)
         size2D = (int(size_multiplier*volume.shape[2]), int(size_multiplier*volume.shape[1]))
 
-        print("spacing: {}".format(spacing))
-        print("volume.shape[0]: {}".format(volume.shape[0]))
+        #print("spacing: {}".format(spacing))
+        #print("volume.shape[0]: {}".format(volume.shape[0]))
 
         z_size = volume.shape[0]
         for z in range(0, z_size):
@@ -632,27 +665,32 @@ class NapariSTPT:
             slice_name = self.slice_names[z+start_slice_number]
 
             spacing = (self.ds1[slice_name].attrs['scale'])
-            print(f"spacing {spacing}")
+            # print(f"spacing {spacing}")
             # fixed.SetSpacing([1.6*spacing[0],1.6*spacing[1]])
             fixed.SetSpacing([resolution*0.1*spacing[1],resolution*0.1*spacing[0]])
 
             transform = sitk.Euler2DTransform()
 
+            align_pos = z + start_slice_number #*self.optical_slices
             alignY = 0
-            if not np.isnan(self.align_y[z]):
-                alignY = -self.align_y[z]*0.1*spacing[1]
+            if not np.isnan(self.corrected_align_y[align_pos]):
+                alignY = -self.corrected_align_y[align_pos]*0.1*spacing[1]
 
             alignX = 0
-            if not np.isnan(self.align_x[z]):
-                alignX = -self.align_x[z]*0.1*spacing[0]
+            if not np.isnan(self.corrected_align_x[align_pos]):
+                alignX = -self.corrected_align_x[align_pos]*0.1*spacing[0]
+
+            #print(f"alignX {alignX}")
+            #print(f"align_pos {align_pos} alignY {self.corrected_align_y[align_pos]}")
 
             transform.SetTranslation([alignY, alignX])
 
             # transform.SetTranslation([-align_y[z]*0.1*spacing[1],-align_x[z]*0.1*spacing[0]])
             # transform.SetTranslation([0.5*-align_y[z],0.5*-align_x[z]])
             # transform.SetTranslation([0,0])
-            print('{}/{}, translate_x: {}, translate_y: {}'.format(z,z_size, alignX, alignY))
-            print('{}/{}, align_x[z]: {}, align_y[z]: {}'.format(z,z_size, self.align_x[z], self.align_y[z]))
+            
+            #print('{}/{}, translate_x: {}, translate_y: {}'.format(z,z_size, alignX, alignY))
+            #print('{}/{}, align_x[z]: {}, align_y[z]: {}'.format(z,z_size, self.align_x[z], self.align_y[z]))
 
             resampler = sitk.ResampleImageFilter()
             # resampler.SetReferenceImage(fixed)
@@ -1073,29 +1111,25 @@ class NapariSTPT:
                 minX):int(maxX), int(minY):int(maxY)]
             self.viewer.layers.remove('C1')
             self.image_translation = (int(minX*output_resolution), int(minY*output_resolution))
-            self.viewer.add_image([self.aligned_1], name='C1', scale=(
-                15, output_resolution, output_resolution), blending='additive', colormap='bop purple')#, translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
+            self.viewer.add_image([self.aligned_1], name='C1', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='bop purple')#, translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
         if self.aligned_2 is not None and self.cb_C2.isChecked() and any(i.name == 'C2' for i in self.viewer.layers):
             self.aligned_2 = self.aligned_2[:, int(
                 minX):int(maxX), int(minY):int(maxY)]
             self.viewer.layers.remove('C2')
             self.image_translation = (int(minX*output_resolution), int(minY*output_resolution))
-            self.viewer.add_image([self.aligned_2], name='C2', scale=(
-                15, output_resolution, output_resolution), blending='additive', colormap='red')#, translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
+            self.viewer.add_image([self.aligned_2], name='C2', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='red')#, translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
         if self.aligned_3 is not None and self.cb_C3.isChecked() and any(i.name == 'C3' for i in self.viewer.layers):
             self.aligned_3 = self.aligned_3[:, int(
                 minX):int(maxX), int(minY):int(maxY)]
             self.viewer.layers.remove('C3')
             self.image_translation = (int(minX*output_resolution), int(minY*output_resolution))
-            self.viewer.add_image([self.aligned_3], name='C3', scale=(
-                15, output_resolution, output_resolution), blending='additive', colormap='green')#, translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
+            self.viewer.add_image([self.aligned_3], name='C3', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='green')#, translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
         if self.aligned_4 is not None and self.cb_C4.isChecked() and any(i.name == 'C4' for i in self.viewer.layers):
             self.aligned_4 = self.aligned_4[:, int(
                 minX):int(maxX), int(minY):int(maxY)]
             self.viewer.layers.remove('C4')
             self.image_translation = (int(minX*output_resolution), int(minY*output_resolution))
-            self.viewer.add_image([self.aligned_4], name='C4', scale=(
-                15, output_resolution, output_resolution), blending='additive', colormap='bop blue')#, translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
+            self.viewer.add_image([self.aligned_4], name='C4', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='bop blue')#, translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
             
             
         self.viewer.layers.remove('Shapes')
@@ -1122,11 +1156,19 @@ class NapariSTPT:
     
     
     def add_polygon(self):
+        self.viewer.window.qt_viewer.update()
 
-        pos = int(self.viewer.cursor.position[0]/15)
+        #print("pos {}".format(self.viewer.dims.point[0]))
+
+
+        
+
+        #pos = float(self.viewer.cursor.position[0]/(float(self.slice_spacing)/float(self.optical_slices)))
+        pos = self.viewer.dims.point[0] / (float(self.slice_spacing)/float(self.optical_slices))
+        print("pos {}".format(pos))
         
         data_length = len(self.viewer.layers['Shapes'].data[0])
-        print(data_length)
+        print("data_length {}".format(data_length))
 
         contour_list = []
         z_pos = self.viewer.layers["Shapes"].data[0][0][0] # z pos
@@ -1143,13 +1185,14 @@ class NapariSTPT:
                 i = i+1
             except:
                 break
+
         contour_list_sorted = sorted(contour_list, key=lambda tup: tup[0])
 
 
 
 
 
-
+        print("contour_list_sorted[0][0] {}".format(contour_list_sorted[0][0]))
         
             
         new_shape = []
@@ -1198,7 +1241,7 @@ class NapariSTPT:
 
         new_shape = np.array(new_shape)
         output_resolution = float(self.pixel_size.text())
-        shapes_layer = self.viewer.add_shapes(new_shape, shape_type='polygon', name = "Shapes", scale=(15, output_resolution, output_resolution),)
+        shapes_layer = self.viewer.add_shapes(new_shape, shape_type='polygon', name = "Shapes", scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution),)
     
 
 
@@ -1522,54 +1565,66 @@ class NapariSTPT:
 
     def SaveVolume(self):
         widget = QtWidgets.QWidget()
-        fname, _ = QtWidgets.QFileDialog.getSaveFileName(
-            widget, 'Save file as', 'c:\\', "Image files (*.tiff *.mha)")
+        if sys.platform == 'linux':
+            fname, _ = QtWidgets.QFileDialog.getSaveFileName(
+                widget, 'Save file as', '/home/', "Image files (*.tiff *.mha)")
+        else:
+            fname, _ = QtWidgets.QFileDialog.getSaveFileName(
+                widget, 'Save file as', 'c:\\', "Image files (*.tiff *.mha)")
 
         if fname != "":
-            print(fname[0])
+            print(fname)
+
+            file_name = fname#"/home/tristan/test.tiff"
 
             output_resolution = float(self.pixel_size.text())
             spacing = (output_resolution, output_resolution,
                        15/self.optical_slices)
 
             # Not the most efficinet method
-            volume = np.zeros(self.shape)
-            volume = np.expand_dims(volume, axis=3)
+            #volume = np.zeros(self.shape)
+            #volume = np.expand_dims(volume, axis=3)
+            volume = []
 
+            #print(f"volume {volume.shape}")
+            if self.cb_C1.isChecked():
+                #volume = np.concatenate((volume, np.expand_dims(self.aligned_1, axis=3)), axis=3)
+                volume.append(self.aligned_1)
+            #except Exception:
+            #   pass
+            if self.cb_C2.isChecked():
+                #volume = np.concatenate((volume, np.expand_dims(self.aligned_2, axis=3)), axis=3)
+                volume.append(self.aligned_2)
+            #except Exception:
+                pass
+            if self.cb_C3.isChecked():
+                #volume = np.concatenate((volume, np.expand_dims(self.aligned_3, axis=3)), axis=3)
+                volume.append(self.aligned_3)
+            #except Exception:
+                pass
+            if self.cb_C4.isChecked():
+                #volume = np.concatenate((volume, np.expand_dims(self.aligned_4, axis=3)), axis=3)
+                volume.append(self.aligned_4)
+            #except Exception:
+            #    pass
+
+            volume = np.array(volume)
             print(f"volume {volume.shape}")
-            try:
-                volume = np.concatenate(
-                    (volume, np.expand_dims(self.aligned_1, axis=3)), axis=3)
-            except Exception:
-                pass
-            try:
-                volume = np.concatenate(
-                    (volume, np.expand_dims(self.aligned_2, axis=3)), axis=3)
-            except Exception:
-                pass
-            try:
-                volume = np.concatenate(
-                    (volume, np.expand_dims(self.aligned_3, axis=3)), axis=3)
-            except Exception:
-                pass
-            try:
-                volume = np.concatenate(
-                    (volume, np.expand_dims(self.aligned_4, axis=3)), axis=3)
-            except Exception:
-                pass
-
+            volume = np.moveaxis(volume,0,3)
             print(f"volume {volume.shape}")
 
-            volume = np.delete(volume, 0, 3)
+            #volume = np.delete(volume, 0, 3)
+            
 
-            print(f"volume {volume.shape}")
+            #print(f"volume {volume.shape}")
+            #100 774 851 4
 
             volume_itk = sitk.GetImageFromArray(volume)
             volume_itk.SetSpacing(spacing)
             caster = sitk.CastImageFilter()
             caster.SetOutputPixelType(sitk.sitkVectorFloat32)
             volume_itk = caster.Execute(volume_itk)
-            sitk.WriteImage(volume_itk, fname[0])
+            sitk.WriteImage(volume_itk, file_name)
 
     def SaveSlice(self):
         widget = QtWidgets.QWidget()
@@ -1615,6 +1670,72 @@ class NapariSTPT:
             sitk.WriteImage(image_itk, fname[0])
 
 
+    def Normalize_slices(self):
+        import statistics
+        print("Normalize_slices")
+        
+
+        output_resolution = float(self.pixel_size.text())
+
+        if self.cb_C1.isChecked():
+            slices, size_x, size_y = self.aligned_3.shape
+            for i in range(0,slices):
+
+                mean = np.mean(self.aligned_1[i,:,:])
+                self.aligned_1[i,:,:] = self.aligned_1[i,:,:] - mean
+
+                st_dev = np.std(self.aligned_1[i,:,:])
+                self.aligned_1[i,:,:] = self.aligned_1[i,:,:] / st_dev
+
+            with napari.gui_qt():
+                self.viewer.layers.remove('C1')
+                self.viewer.add_image([self.aligned_1], name='C1', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='bop purple')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+
+
+        if self.cb_C2.isChecked():  
+            slices, size_x, size_y = self.aligned_3.shape
+            for i in range(0,slices):
+
+                mean = np.mean(self.aligned_2[i,:,:])
+                self.aligned_2[i,:,:] = self.aligned_2[i,:,:] - mean
+
+                st_dev = np.std(self.aligned_2[i,:,:])
+                self.aligned_2[i,:,:] = self.aligned_2[i,:,:] / st_dev
+
+            with napari.gui_qt():
+                self.viewer.layers.remove('C2')
+                self.viewer.add_image([self.aligned_2], name='C2', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='red')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+
+        if self.cb_C3.isChecked():
+            slices, size_x, size_y = self.aligned_3.shape
+            for i in range(0,slices):
+
+                mean = np.mean(self.aligned_3[i,:,:])
+                self.aligned_3[i,:,:] = self.aligned_3[i,:,:] - mean
+
+                st_dev = np.std(self.aligned_3[i,:,:])
+                self.aligned_3[i,:,:] = self.aligned_3[i,:,:] / st_dev
+
+            with napari.gui_qt():
+                self.viewer.layers.remove('C3')
+                self.viewer.add_image([self.aligned_3], name='C3', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='green')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+
+        if self.cb_C4.isChecked():
+            slices, size_x, size_y = self.aligned_4.shape
+            for i in range(0,slices):
+
+                mean = np.mean(self.aligned_4[i,:,:])
+                self.aligned_4[i,:,:] = self.aligned_4[i,:,:] - mean
+
+                st_dev = np.std(self.aligned_4[i,:,:])
+                self.aligned_4[i,:,:] = self.aligned_4[i,:,:] / st_dev
+
+            with napari.gui_qt():
+                self.viewer.layers.remove('C4')
+                self.viewer.add_image([self.aligned_4], name='C4', scale=(float(self.slice_spacing)/float(self.optical_slices), output_resolution, output_resolution), blending='additive', colormap='bop blue')#, translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
+
+            
+
     def main(self):
 
         with napari.gui_qt():
@@ -1672,6 +1793,11 @@ class NapariSTPT:
             #hbox.addWidget(self.comboBoxResolution)
             #hbox.addStretch(1)
 
+            hbox.addWidget(QtWidgets.QLabel("Slice spacing:"))
+            self.m_slice_spacing = QtWidgets.QLineEdit("15")
+            self.m_slice_spacing.setMaximumWidth(50)
+            hbox.addWidget(self.m_slice_spacing)
+
             hbox.addWidget(QtWidgets.QLabel("Output pixel size:"))
             self.pixel_size = QtWidgets.QLineEdit("15")
             self.pixel_size.setMaximumWidth(50)
@@ -1695,14 +1821,14 @@ class NapariSTPT:
             hbox.addStretch(1)
             vbox.addLayout(hbox)
 
-            hbox = QtWidgets.QHBoxLayout()
-            hbox.addWidget(QtWidgets.QLabel(
-                "Maximum number of optical slices:"))
-            self.nr_optical_slices = QtWidgets.QLineEdit("1")
-            self.nr_optical_slices.setMaximumWidth(50)
-            hbox.addWidget(self.nr_optical_slices)
-            hbox.addStretch(1)
-            vbox.addLayout(hbox)
+            # hbox = QtWidgets.QHBoxLayout()
+            # hbox.addWidget(QtWidgets.QLabel(
+            #     "Maximum number of optical slices:"))
+            # self.nr_optical_slices = QtWidgets.QLineEdit("1")
+            # self.nr_optical_slices.setMaximumWidth(50)
+            # hbox.addWidget(self.nr_optical_slices)
+            # hbox.addStretch(1)
+            # vbox.addLayout(hbox)
 
             hbox = QtWidgets.QHBoxLayout()
             hbox.addWidget(QtWidgets.QLabel("Load channels:"))
@@ -1809,6 +1935,11 @@ class NapariSTPT:
             bSaveVolume.clicked.connect(self.SaveVolume)
             hbox.addWidget(bSaveVolume)
 
+            bNormalize = QtWidgets.QPushButton('Normalize')
+            bNormalize.setCheckable(True)
+            bNormalize.clicked.connect(self.Normalize_slices)
+            hbox.addWidget(bNormalize)
+
             #bSaveMovie = QtWidgets.QPushButton('Save movie')
             #bSaveMovie.setCheckable(True)
             #bSaveMovie.clicked.connect(self.SaveMovie)
@@ -1849,6 +1980,10 @@ class NapariSTPT:
             animation_widget = AnimationWidget(self.viewer)
             self.viewer.window.add_dock_widget(animation_widget, area='right')
 
+            #@viewer.bind_key('z')
+            #def print_z_slice(viewer):
+            #    print(viewer.dims.point[0])
+
             # inputfile = ''
             # try:
             #     opts, args = getopt.getopt(
